#include <windows.h>
#include <GL/glut.h>
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <stdio.h>

CGcontext context;  
CGprofile fragmentProfile, vertexProfile;
CGprogram fragmentProgram, vertexProgram;

CGparameter matAmbient, matDiffuse, matSpecular, matShininess;
CGparameter lightAmbient, lightDiffuse, lightSpecular;
CGparameter lightPosition;
CGparameter eyePosition;
CGparameter attenuation;
CGparameter mv;
CGparameter mvit;

static void checkForCgError(const char *situation)
{
  CGerror error;
  const char *string = cgGetLastErrorString(&error);

  if (error != CG_NO_ERROR) {
    printf("%s: %s\n", situation, string);
    if (error == CG_COMPILER_ERROR) {
      printf("%s\n", cgGetLastListing(context));
    }
    exit(1);
  }
}

float identityMatrix[16] = {1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1};

void display()
{
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	cgGLEnableProfile(vertexProfile);
	cgGLEnableProfile(fragmentProfile);
	cgGLBindProgram(vertexProgram);
	cgGLBindProgram(fragmentProgram);

	cgSetMatrixParameterfr(mv, identityMatrix);
	cgSetMatrixParameterfr(mvit, identityMatrix);
	cgUpdateProgramParameters(vertexProgram);

	cgSetParameter3f(matAmbient, 0.1f, 0.1f, 0.1f);
	cgSetParameter3f(matDiffuse, 0.1f, 0.8f, 0.8f);
	cgSetParameter3f(matSpecular, 1, 1, 1);
	cgSetParameter1f(matShininess, 10);

	cgSetParameter3f(lightAmbient, 1, 1, 1);
	cgSetParameter3f(lightDiffuse, 1, 1, 1);
	cgSetParameter3f(lightSpecular, 1, 1, 1);
	cgSetParameter3f(lightPosition, 0, 1, 1.5);

	cgSetParameter3f(attenuation, 1, 0, 0);
	cgSetParameter3f(eyePosition, 0, 0, 5);
	cgUpdateProgramParameters(fragmentProgram);	

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, 1, 0.1, 1000);		
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();		
	gluLookAt(0,0,5,0,0,0,0,1,0);

	glutSolidSphere(1, 50, 50);
		
	cgGLDisableProfile(fragmentProfile);
	cgGLDisableProfile(vertexProfile);

	glutSwapBuffers();
}

#define GET_PARAMETER(__name) \
	__name = cgGetNamedParameter(fragmentProgram, #__name); \
	checkForCgError("getting " #__name " parameter");


void initCg()
{
	context = cgCreateContext();
	checkForCgError("creating context");
	cgGLSetDebugMode( CG_FALSE );
	cgSetParameterSettingMode(context, CG_DEFERRED_PARAMETER_SETTING);

	vertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgGLSetOptimalOptions(vertexProfile);
	checkForCgError("selecting vertex profile");

	fragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(fragmentProfile);
	checkForCgError("selecting fragment profile");

	vertexProgram = cgCreateProgramFromFile(
		context,
		CG_SOURCE,
		"vertex-program.cg",
		vertexProfile,
		"main",
		NULL);
	checkForCgError("creating vertex program");
	cgGLLoadProgram(vertexProgram);
	checkForCgError("loading vertex program");

	mv = cgGetNamedParameter(vertexProgram, "mv");
	checkForCgError("creating fragment program");
	mvit = cgGetNamedParameter(vertexProgram, "mvit");
	checkForCgError("creating fragment program");

	fragmentProgram = cgCreateProgramFromFile(
		context,
		CG_SOURCE,
		"fragment-program.cg",
		fragmentProfile,
		"main",
		NULL);
	checkForCgError("creating fragment program");
	cgGLLoadProgram(fragmentProgram);
	checkForCgError("loading fragment program");

	GET_PARAMETER(matAmbient);
	GET_PARAMETER(matDiffuse);
	GET_PARAMETER(matSpecular);
	GET_PARAMETER(matShininess);
	GET_PARAMETER(lightAmbient);
	GET_PARAMETER(lightDiffuse);
	GET_PARAMETER(lightSpecular);
	GET_PARAMETER(lightPosition);
	GET_PARAMETER(attenuation);
	GET_PARAMETER(eyePosition);
}

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(512, 512);
	glutCreateWindow("Simple Vertex Program");

	initCg();

	glEnable(GL_DEPTH_TEST);
	glutDisplayFunc(display);

	glutMainLoop();
}
