#include <windows.h>
#include <GL/glut.h>
#include <IL/il.h>
#include <IL/ilu.h>
#include <Cg/cg.h>
#include <Cg/cgGL.h>
#include <stdio.h>

CGcontext context;  
CGprofile fragmentProfile;
CGprogram fragmentProgram;

CGparameter leftSeparation;
CGparameter rightSeparation;
CGparameter texture;

GLuint texID;

static void checkForCgError(const char *situation)
{
  CGerror error;
  const char *string = cgGetLastErrorString(&error);

  if (error != CG_NO_ERROR) {
    printf("%s: %s\n", situation, string);
    if (error == CG_COMPILER_ERROR) {
      printf("%s\n", cgGetLastListing(context));
    }
    exit(1);
  }
}

float identityMatrix[16] = {1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1};

void display()
{
	glClearColor(0,0,0,0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	cgGLEnableProfile(fragmentProfile);
	cgGLBindProgram(fragmentProgram);

	cgGLSetTextureParameter(texture, texID);
	cgGLEnableTextureParameter(texture);

	glBegin(GL_TRIANGLES);
		glTexCoord2f(0, 0);
		glVertex2f(-0.8, 0.8);

		glTexCoord2f(1, 0);
		glVertex2f(0.8, 0.8);

		glTexCoord2f(0.5, 1);
		glVertex2f(0.0, -0.8);
	glEnd();
		
	cgGLDisableProfile(fragmentProfile);

	glutSwapBuffers();
}

void initCg()
{
	context = cgCreateContext();
	checkForCgError("creating context");
	cgGLSetDebugMode( CG_FALSE );
	cgSetParameterSettingMode(context, CG_DEFERRED_PARAMETER_SETTING);

	fragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(fragmentProfile);
	checkForCgError("selecting fragment profile");

	fragmentProgram = cgCreateProgramFromFile(
		context,
		CG_SOURCE,
		"fragment-program.cg",
		fragmentProfile,
		"main",
		NULL);
 	checkForCgError("creating fragment program");
	cgGLLoadProgram(fragmentProgram);
	checkForCgError("loading fragment program");

	texture = cgGetNamedParameter(fragmentProgram, "texture");
}

void initTexture()
{
	glGenTextures(1, &texID);
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	ILuint image;
	ilGenImages(1, &image);
	ilBindImage(image);
	ilLoadImage((wchar_t *)"../images/patchouli.jpg");	
	ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE);

	glTexImage2D(GL_TEXTURE_2D, 0, ilGetInteger(IL_IMAGE_BPP), ilGetInteger(IL_IMAGE_WIDTH), ilGetInteger(IL_IMAGE_HEIGHT), 0,
		ilGetInteger(IL_IMAGE_FORMAT), GL_UNSIGNED_BYTE, ilGetData());
	
	ilDeleteImages(1, &image);
}

int main(int argc, char **argv)
{
	ilInit();
	iluInit(); 

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(512, 512);
	glutCreateWindow("Simple Texture");

	initTexture();
	initCg();

	glEnable(GL_DEPTH_TEST);
	glutDisplayFunc(display);

	glutMainLoop();
}
